function [heights60,dates60]=filtto60min(heights,dates,flag)

% FILTTO60MIN - Filtering from 5 min or longer to 1 hour, preserving tides
%
%Filtering from 5 min or longer to 1 hour
%using various filters designed to avoid change to tidal power.
%The intention is that the main tidal frequencies should NOT be attenuated
%by these filters.
%
%Most of these filters are documented in Pugh 1987 (the details are not
%included in Pugh & Woodworth book). The coefficients are taken from the
%inhouse Fortran code filter5min_new_flag.f95 and cross-checked with Pugh 1987.
%
%All the filters are intended to be symmetrical so no phase shifts are
%introduced. The 5,10,15 min filters are designed to have the same
%characteristics. The 30 min filter has a sharper cut-off. For M8 and
%higher harmonics there is significant attentuation, so for shallow water
%if may be necessary to work at the original sampling interval. However it
%is unlikely that this will have a substantial effect on monthly mean SL.
% See also Godin 1972.
%
%Filters in this file:
% filter time length is between first point and last
%
% timedelta         : Name in Fortran code : points : length
%
% 5 min   to hourly : VASSI5 (Vassie)  : 2*54+1 : 9 hour
% 6 min   to hourly : FIL18  (Vassie?) : 2*39+1 : 7 hour 48 min
% 7.5 min to hourly : FIL17B (Vassie?) : 2*31+1 : 7 hour 45 min
% 10 min  to hourly : PUGH10 (Pugh)    : 2*27+1 : 9 hour
% 15 min  to hourly : FIL03B (Vassie)  : 2*15+1 : 10 hour
% 30 min  to hourly : PUGH30 (Pugh)    : 2*12+1 : 12 hour
% 60 min  to hourly : Will return the same data
%
% The interval of the input dates (matlab datenum) is used to
% automatically select the appropriate filter function
% If the interval varies (tolerance 10 sec) or is more than 10 sec
% different to one of these, then there will be an error.
%
% If the timedelta<5 minutes, there is an initial step of taking 5 or 6
% minute means. For this step flags are used, and it is assumed that any
% data flagged *4* is to be discarded. Gaps are filled up to a maximum of
% PERMITTEDGAP = 10 minutes. 
%
% Option to take flags exists but FLAGS ARE NOT YET IMPLEMENTED!
%
% For filtering from very high frequency to 5 or 6 minutes, use NMINMEANS
%
% Dates are output on the hour
%
% Matlab version Joanne Williams joll@noc.ac.uk 2018

%See
%Ian Vassie's code - see /work/anhi/code/filt5m


%Ensure input(s) are column vectors, if not output is in the same format
if isrow(heights)
    returnrowheights=1;
else
    returnrowheights=0;
end

if isrow(dates)
    returnrowdates=1;
else
    returnrowdates=0;
end

dates=dates(:);
heights=heights(:);


if nargin<3
    flag=[];
end

if ~isempty(flag)
    warning('flagged data not handled in filter to hourly data')
end

%Set tolerance on how accurate time data needs to be.
tol=10/60; %10 seconds 

%Set permitted gap in data at higher frequency than 5 minutes, that will be
%averaged to 5 minute data. Gap may be due to flagging or nan
permittedgap=10; %10 minutes

%Estimate interval of data. Use of median allows occasional gaps. 
%No allowance is made for a large chunk of data at a different interval -
%it will probably fail later. Highly irregular data will probably fail later. 
timedelta=median(diff(dates))*24*60; %time interval in minutes


if abs(timedelta-60)<tol  %hourly
    %No change
    %dates=dates;
    %heights=heights;
    XX=60;
elseif abs(timedelta-30)<tol
    XX=30;
elseif abs(timedelta-15)<tol
    XX=15;
elseif abs(timedelta-10)<tol
    XX=10;
elseif abs(timedelta-7.5)<tol
    XX=7.5;
elseif abs(timedelta-6)<tol
    XX=6;
elseif abs(timedelta-5)<tol
    XX=5;
elseif timedelta<5
    
    % If the data starts off at higher frequency than 5 minute,
    % then start by averaging to 5 or 
    % 6 minutes (only for initial 2 or 3 minute data)
    if abs(timedelta-2)<tol||abs(timedelta-3)<tol
        
        [heights,dates]=Nminmeans(heights,dates,flag,6,permittedgap);
        warning('FLAGS!')
        XX=6; timedelta=6;
    else
        [heights,dates]=Nminmeans(heights,dates,flag,5,permittedgap);
        warning('FLAGS!')
        XX=5;timedelta=5;
    end
    
else
    error('Unrecognised time interval')
end


%Requires uniformly spaced data (unless higher than 5 min input and already
%corrected). 
%Check there are no variations in time stamp (allow tolerance eg 10sec) and
%nudge time stamp to be exact


if any(abs(diff(dates)*60*24-XX)>tol) %Any of the date spaces more than (tol=10sec) different from timedelta
    if all(abs(modnear(diff(dates)*60*24,0,timedelta)))<tol %eg 5+eps,10-eps,20+eps,25+eps,
        %indicates there are probably gaps to fill.
        datesq=dates(1):XX/(60*24):dates(end); %XX is in minutes!
        datesq=rounddatesec(datesq,1);
        hq = interp1(dates,heights,datesq,'nearest');
        
        %identify which dates are big gaps & set as nan;
        % Allow datesq to be permittedgap/2 minutes away from an unflagged data point
        inewdate=~ismembertol(datesq,dates,permittedgap/2/(24*60),'datascale',1);
        hq(inewdate)=nan;
        
        heights=hq(:);
        dates=datesq(:);
    else 
%Such variations *might* be caused by having gaps in the time stamp instead of NaN. 
%If so, fill the gaps. (Not implemented)
        error('filtto60min:Requires uniformly spaced data')
    end
end



%plot(dates(isnan(heights)),0,'^')
[heights60,dates60]=filtXXminto60min(heights,dates,[],XX);

%plot(dates60,heights60,'+')





if returnrowdates
    dates60=dates60(:)';
end
if returnrowheights
    heights60=heights60(:)';
end


end

%% Subfunction: selection of filters according to spacing XX
function [heightsout,datesout]=filtXXminto60min(heightsin,datesin,flag,XX)

switch XX
    case 60    %No change
        heightsout=heightsin;
        datesout=datesin;
        return
    case 30
        %     ! PUGH30 FILTER TAKEN FROM APPENDIX 1 OF PUGH(1987) FOR FILTERING
        % ! 30 MINUTE DATA TO HOURLY VALUES.
        
        wts30=[0.3750000,...
            0.2898969, 0.1062099, -0.0355647, -0.0625000, -0.0164860,...
            0.0208333,...
            0.0179817, 0.0000000,-0.0061020,-0.0020433, 0.0002740,...
            0.0];
        wts=[wts30(end:-1:2),wts30];
    case 15
        %     ! 03B FILTER (INVENTED I BELIEVE BY IAN VASSIE) FOR FILTERING
        % ! 15 MINUTE DATA TO HOURLY VALUES. THIS FILTER IS USED BY BODC
        % ! FOR DATA PROCESSING.
        wts15=[...
            0.21875000, 0.20031524, 0.15112494, 0.08694117,...
            0.02667355,-0.01496743,...
            -0.03233332,-0.02929314,-0.01562501,-0.00159472, 0.00644497,...
            0.00753492, 0.00457646, 0.00129718,-0.00023659,-0.00023321 ];
        
        wts=[wts15(end:-1:2),wts15];
    case 10
        % ! PUGH10 FILTER TAKEN FROM APPENDIX 1 OF PUGH(1987) FOR FILTERING
        % ! 10 MINUTE DATA TO HOURLY VALUES.
        
        wts10=[0.1296296,...
            0.1257208, 0.1144630, 0.0971907, 0.0759010, 0.0529545,...
            0.0307322,...
            0.0113057,-0.0038254,-0.0138889,-0.0188692,-0.0194147,...
            -0.0166494,...
            -0.0119285,-0.0065884,-0.0017344, 0.0018986, 0.0039967,...
            0.0046296,...
            0.0041458, 0.0030310, 0.0017660, 0.0007136, 0.0000589,...
            -0.0001939,...
            -0.0001725,-0.0000576, 0.0];
        wts=[wts10(end:-1:2),wts10];
    case 7.5
        % ! 17B FILTER (INVENTED I BELIEVE BY IAN VASSIE) FOR FILTERING
        % ! 7.5 MINUTE DATA TO HOURLY VALUES.
        
        wts075=[...
            0.10937500,...
            0.10702016, 0.10015762, 0.08937112, 0.07556247,...
            0.05985418, 0.04347059,...
            0.02761198, 0.01333678, 0.00146538,-0.00748372,...
            -0.01331978,-0.01616666,...
            -0.01641475,-0.01464657,-0.01154725,-0.00781251,...
            -0.00406622,-0.00079736,...
            0.00167733, 0.00322249, 0.00386721, 0.00376746,...
            0.00315586, 0.00228823,...
            0.00139555, 0.00064859, 0.00013954,-0.00011830,....
            -0.00017484,-0.00011661,...
            -0.00003547 ];
        
        wts=[wts075(end:-1:2),wts075];
        
    case 6
        % ! 18 FILTER (INVENTED I BELIEVE BY IAN VASSIE) FOR FILTERING
        % ! 6 MINUTE DATA TO HOURLY VALUES.
        wts06=[...
            0.08750000,...
            0.08629118, 0.08273135, 0.07701568, 0.06945417,...
            0.06044997, 0.05047158,...
            0.04002100, 0.02960042, 0.01968006, 0.01066942,...
            0.00289423,-0.00341946,...
            -0.00815316,-0.01129538,-0.01293332,-0.01323771,...
            -0.01244252,-0.01082152,...
            -0.00866373,-0.00625000,-0.00383248,-0.00161860,...
            0.00023951, 0.00164958,...
            0.00257799, 0.00304341, 0.00310632, 0.00285590,...
            0.00239584, 0.00183058,...
            0.00125346, 0.00073771, 0.00033121, 0.00005497,...
            -0.00009463,-0.00014061,...
            -0.00011804,-0.00006581,-0.00001856 ...
            ];
        
        wts=[wts06(end:-1:2),wts06];
        
    case 5
        
        % filter is 9 hours long
        % ! VASSI5 FILTER TAKEN FROM IAN VASSIE'S ARCHIVE FOR FILTERING
        % ! 5 MINUTE DATA TO HOURLY VALUES.
        % via filter5min_new_flag.f95
        
        wts05=[0.06481481, 0.06432247, 0.06286039,...
            0.06047279, 0.05723149, 0.05323311, 0.04859535,...
            0.04345253, 0.03795049, 0.03224121, 0.02647724,...
            0.02080628, 0.01536610, 0.01027997, 0.00565285,...
            0.00156847, -0.00191270, -0.00475439, -0.00694444,...
            -0.00849382, -0.00943458, -0.00981725, -0.00970737,...
            -0.00918181, -0.00832469, -0.00722331, -0.00596423,...
            -0.00462963, -0.00329418, -0.00202249, -0.00086722,...
            0.00013207, 0.00094930,  0.00157160,  0.00199837,...
            0.00223976, 0.00231481, 0.00224920, 0.00207288,...
            0.00181778, 0.00151551, 0.00119541, 0.00088297,...
            0.00059858, 0.00035681, 0.00016620, 0.00002943,...
            -0.00005603, -0.00009697, -0.00010319, -0.00008622,...
            -0.00005783, -0.00002879, -0.00000767, 0.0];
        wts=[wts05(end:-1:2),wts05];
        
    otherwise
        error('Unrecognised time stamp')
        
end

if ~isempty(flag)
    warning('flagged data not handled in filter')
end

Nt=length(datesin);

%filter is wts eg    wts=[wts05(end:-1:2),wts05];
Nfilt=length(wts);
Nhalf=(Nfilt-1)/2; %exclude central value

%List of hours to return that fall in the date range.
datesHR=[ceil(datesin(1)*24):floor(datesin(end)*24)]/24;

%Indices of datesin nearest to each hour.
itHR=round(interp1(datesin,1:Nt,datesHR));

NHr=length(itHR);

datesout=datesHR;
heightsout=nan(NHr,1);
for ihr=1:NHr
    it=itHR(ihr); %this is the central value
    
    if it<=Nhalf||it>=(Nt-Nhalf+1)
        heightsout(ihr)=nan;
    else
        %multiply weights by heights %use (:) to ensure size consistent
        heightsout(ihr)=sum(wts(:).*heightsin(it+[-Nhalf:Nhalf]));
        
    end
    
end


end