function [hhourly,dateshourly]=hourlymeans(heightsin,datesin,offset,usenanmean)

% HOURLYMEANS Simple hourly means - no tidal filtering.
% To filter avoiding tidal bias, use filtto60min.m
%
%
% Returns dates at the start of the hour over which they are averaged
% So use offset=1/48 (ie half an hour) to return dates at 00:30
% Or dates+1/48,  offset=0 to average symmetrically around the 00:00
% (I know this seems weird but it's to be consistent with behaviour of
% dailymeans.m which expects you to want to return dates as days)
%
% [HHOURLY,DATESHOURLY]=HOURLYMEANS(HEIGHTSIN,DATESIN,OFFSET,USENANMEAN)
%
% OFFSET is given in days
%
% Set
% USENANMEAN=1 to ignore nans,
% USENANMEAN=0 to allow nans to propogate
% USENANMEAN>1 to set the minimum number of non-nan points to average.
% (eg USENANMEAN=30 on one minute data to require at least half the data)
%
% HOURLYMEANS is designed to cope with irregular and missing data. 
% Problems can be caused by use of low frequency data - eg 30 minute.
% Because no account is taken of the position of the data within the hour's 
% window, the phase of the data can end up changing. This would be the benefit of
% windowing beyond the hourly interval. 
% 
%
% Joanne Williams, joll@noc.ac.uk, 2018

%Cope with empty input.
if isempty(datesin)||isempty(heightsin)
    dateshourly=[];
    hhourly=[];
    return
end

%Ensure input(s) are column vectors, if not output is in the same format
if isrow(heightsin)
    returnrowheights=1;
else
    returnrowheights=0;
end

if isrow(datesin)
    returnrowdates=1;
else
    returnrowdates=0;
end

datesin=datesin(:);
heightsin=heightsin(:);

if nargin<3, offset=0; end

if nargin<4, usenanmean=0; end


[Y,MO,D,H,~,~] = datevec(datesin);

[~,ia,ic]=unique([Y,MO,D,H],'rows','first');
%ia is an index of all hours containing any data
%ic is an index of all data hours
dateshourly=floor(datesin(ia)*24)/24+offset; %output dates at start of hour + offset

%usenanmean=0 -> [1 nan nan 4 ..60]-> nan
%usenanmean=1 -> [1 nan nan 4 ..60]-> mean [1 4 ..60]
%Any number of non-nanpoints will count
%usenanmean=10 -> requires at least 10 points to be non-nan

Nhm=length(dateshourly);
hhourly=nan(Nhm,1);
if usenanmean==0
    %     for i=1:Nhm
    %         hhourly(i)=mean(heightsin(ic==i));
    %     end
    sumprev=0;
    icprev=ic(1);
    Nprev=0;
    
    for j=1:length(ic)
        if ic(j)==icprev
            
            sumprev=sumprev+heightsin(j);
            Nprev=Nprev+1;
            
        else
            hhourly(ic(j-1))=sumprev/Nprev;
            sumprev=heightsin(j);
            Nprev=1;
            icprev=ic(j);
        end
    end
    
elseif usenanmean==1
    
    %This is the same as the following code, but *slower* than the loop
    %     for i=1:Nhm
    %         htemp=heightsin(ic==i);
    %         hhourly(i)=mean(htemp,'omitnan');
    %     end
    sumprev=0;
    icprev=ic(1);
    Nprev=0;
    
    for j=1:length(ic)
        if ic(j)==icprev
            if ~isnan(heightsin(j))
                sumprev=sumprev+heightsin(j);
                Nprev=Nprev+1;
            end
        else
            hhourly(ic(j-1))=sumprev/Nprev;
            sumprev=heightsin(j);
            Nprev=1;
            icprev=ic(j);
        end
    end
    
elseif usenanmean>1
    %     for i=1:Nhm
    %         inan=isnan(heightsin(ic==i));
    %         if sum(~inan)>usenanmean %aka minpoints
    %             hhourly(i)=mean(heightsin(ic==i),'omitnan');
    %         else
    %
    %         end
    %     end
    sumprev=0;
    icprev=ic(1);
    Nprev=0;
    
    for j=1:length(ic)
        if ic(j)==icprev
            if ~isnan(heightsin(j))
                sumprev=sumprev+heightsin(j);
                Nprev=Nprev+1;
            end
        else
            if Nprev>usenanmean
                hhourly(ic(j-1))=sumprev/Nprev;
            else
                hhourly(ic(j-1))=nan;
            end
            sumprev=heightsin(j);
            Nprev=1;
            icprev=ic(j);
        end
    end
    
    
    
else
    error('Argument unknown for usenanmean')
end


if returnrowdates
    dateshourly=dateshourly(:)';
end
if returnrowheights
    hhourly=hhourly(:)';
end