function [s,h,p,EN,p1] = longfindfast(mjdn)

% LONGFINDFAST     Calculate astronomical arguments for tides
%     This version from Chris Hughes 28 June 2002
% converted to Matlab  Simon Williams 2018
% 
% =================================================================
%
% Input: mjdn = modified julian day number
%        iout = integer unit for output (0 produces no output)
% Output: s = mean longitude of Moon 
%         h = mean longitude of Sun
%         p = mean longitude of lunar perigee
%        EN = mean longitude of lunar node (point where plane of
%             lunar orbit crosses plane of equator)
%        p1 = mean longitude of perihelion
%
%        All in degrees,  calculated at 00:00 on that day.

%
%
% Time is calculated internally from zero at 12:00 1-Jan-2000 TDT
% Longitudes at t=0 and formulae for other times taken from
% Cartwright, D.E., 1985: Tidal Prediction and Modern Time Scales,
% International Hydrographic Review, Monaco, 62(1), 127-138.
%
% The formulae are for time (TT) in (atomic clock) seconds, converted to
% Julian centuries (86400*365.25*100 s = 1 Julian century). Due to
% variations in earth rotation rate, this differs from civil time by
% an amount which increases with distance from the reference time - by
% less than a minute within a century, increasing to about 6 hours at
% 750 BC. 6 hours in the position of the moon translates to an M2 phase
% shift of about 6 degrees. They are part of a Taylor series and will
% drift from reality at long time scales (centuries).
% 
% To account for this, two time variables are used:
%
% It is assumed that the measurement time of day is t = civil time (UTC, less than
%    a second from UT1 which is actually a measure of earth rotation)
%
% tt = Terrestrial Time (the scale used to derive the position of the sun
%    and moon) is the actual (atomic) time elapsed. Units of tt are Julian centuries
%    (36525*86400 seconds) measured relative to 12:00 1 Jan 2000 (J2000)
%
% A quadratic correction is applied to convert from t to tt: deltat = tt - t,
%   where deltat = a + b*tt + c*tt^2 (in practice using t instead of tt)
%
% c is 31.0 (taken from Ch 14 of Stephenson: Historical eclipses and earth rotation)
%   this corresponds to a lengthening of length of day at a rate 1.7 ms per century
%   (2*36525*31 = 0.001697)
%
% b is 90.0, chosen to ensure a good fit to measured deltat since 1900 and
%   reasonable fit to earlier astronomical observations and eclipse data going
%   back to 750 BC. This gives an excellent fit for 1940-2000 (2-3 seconds)
%   but the error rises to ~10 s earlier in 20th century and by late 2015.
%   Errors remain within about 30 minutes for times back to 750 BC.
%
% a is chosen by insisting that deltat = 32.184 s at t0 = 00:00 1 Jan 1958,
%   since that was the actual value of deltat when TAI was set equal to UT2
%   (an approximation of UT1 with seasonal terms removed). Since
%   mjd of this time is 36204.0, and mjd of J2000 is 51544.5d0, this gives
%   t0 = (36204.0d0-51544.5d0)/36525.d0 in Julian centuries, and hence
%   a = 32.184d0 - b*t0 - c*t0**2
%
% The size of errors here (seconds to minutes) should only be interpreted
% in terms of the distance moved by the sun and moon relative to the stars
% over that period. The biggest issue is the moon, which moves at about 0.55
% degrees per hour. 
%
%.
      cycle=360.0;
      c = 32.0;
      b = 90.0;
      t0 = (36204.0-51544.5)/36525.0;
      a = 32.184 - b * t0 - c * t0.^2;
%
% calculate t (julian centuries UTC after 12:00, 1 Jan 2000)
% 51544 = mjdn of 1 Jan 2000
%.

      t = (mjdn-51544-0.5)./36525;

%
% calculate correction to t for earth rotation variations and
% difference between TDT and TAI (TDT-TAI=32.184s)
%     (dt in seconds, set dt=32.284s at 00:00 1-Jan-1958)
%.
%     mjdn for that date is 36204
%.
%      t0 = (36204.0d0-51544.5d0)/36525.d0
%      c = 31.0d0
%      b = 90.0d0
%      a = 32.184d0 - b*t0 - c*t0**2

      dt = a + b * t + c * t.^2;

      tt=t + dt./(86400.0*36525.0);
%.
      s   = 218.3166 + 481267.8811.*tt - 0.0019.*tt.^2;
      h   = 280.4661 +  36000.7698.*tt + 0.0003.*tt.^2;
      p   =  83.3532 +   4069.0136.*tt - 0.0106.*tt.^2;
      EN  = 125.0445 -   1934.1364.*tt + 0.0018.*tt.^2;
      p1  = 282.9384 +      1.7194.*tt + 0.0002.*tt.^2;
%.
      s  = mod( s,cycle);
      h  = mod( h,cycle);
      p  = mod( p,cycle);
      EN = mod(EN,cycle);
      p1 = mod(p1,cycle);

      I = s < 0;  s(I)  = s(I)  + cycle;
      I = h < 0;  h(I)  = h(I)  + cycle;
      I = p < 0;  p(I)  = p(I)  + cycle;
      I = EN < 0; EN(I) = EN(I) + cycle;
      I = p1 < 0; p1(I) = p1(I) + cycle;
%.
%.
end
