
function [sensornames]=readIOC(sitecode,dateS,dateE,datadir,mode)

%READIOC Read data from IOC site between given dates
%The webservice is limited to 30 days at a time, so read chunks
%and concatenate matching channels.
%
% Output a cell array containing the sensor names.
%
%Save each channel in its own file eg
%/work/dataQC/data/calq/rad_dateS_dateE.txt
%with dates,levels.
%
% Each contains a stream for each separate sensor.
%
% Dates dowloaded are dateS <= date < dateE 
% ie from 00:00 on dateS 
% to 23:59 on the day before dateE
%
% Eg to download all data in 2013:
% dateS=datenum(2013,01,01);
% dateE=datenum(2014,01,01);
%
% datadir='/work/dataQC/data/calq/';
% sitecode='calq';
%
% MODE can be 'interactive', if you wish to be able to overwrite existing
% data, or 'auto'
%
% This function is required for TIDEGAUGEQC.
% 
% Joanne Williams, joll@noc.ac.uk, Oct 2018

%Direct read example:
%http://www.ioc-sealevelmonitoring.org/service.php?query=data&code=calq&format=ascii&timestart=2013-12-12T12:29&timestop=2013-12-22T12:29


if nargin<5 %Original option
    mode='interactive'; %vs 'auto'
end


%check input - is the date plausible
if datenum(1700,01,01)>dateS||datenum(1700,01,01)>dateE||...
        datenum(2200,01,01)<dateS||datenum(2200,01,01)<dateE
    error('Unexpected date - check format?')
end

genericfilename=[datadir,sitecode,'_*' ...
    datestr(dateS,'_yyyymmdd')...
    datestr(dateE,'_yyyymmdd')...
    '.txt']; 

if ~isempty(dir(genericfilename))
    disp(genericfilename)
    disp('Some files already exist for this location and dates!')
    
    if strcmpi(mode,'auto')
        %Skip download and Go straight to reload if possible
            reply='S';
    else
            disp('Do you need to download?')
            disp('You may prefer instead to reload them.')
            %reply = input('Continue with download? Y/N [N]:','s');
            reply = input('Download anyway / Skip download / Abort? D/[S]/A :','s');
    end
    
    if strcmpi(reply,'D')
        disp('OK, downloading...')
    elseif strcmpi(reply,'A')
        error('Aborting')
    else
        sensornames={};
        disp('Skipping download.')
        return
    end
    
end

%IOC site will only allow 30 days download at a time, so break into chunks
for idownloadchunk=0:floor((dateE-dateS)/30)
    
    
    datepartS=dateS+30*idownloadchunk;
    datepartE=min(datepartS+30,dateE)-1/(24*60); %minus 1 minute - note this is the precision of the IOC site interface
    timestartstr=datestr(datepartS,'yyyy-mm-ddTHH:MM');
    timeendstr=datestr(datepartE,'yyyy-mm-ddTHH:MM');
    
    textdownloaded=webread('http://www.ioc-sealevelmonitoring.org/service.php?',...
        'query','data','format','ascii',...
        'code',sitecode,...
        'timestart',timestartstr,...
        'timestop',timeendstr,weboptions('Timeout',20));
    
    %If this download period is empty (long data gap) then skip to the next
    if isempty(textdownloaded)
        continue
    end
    
    %% Scan content of test and parse it
    C=textscan(textdownloaded,'%f%10c%8c%s','headerlines',1,'whitespace',' \n\r\t');
    
    %slevel=C{1};
    %sdate=datenum([C{2},C{3}],'yyyy-mm-ddHH:MM:SS');
    sensor=C{4};
    
    Ntot=length(sensor);
    
    %% Separate sensor streams (automatically detect sensors)
    sensortag=ones(Ntot,1);
    sensorcount=1;
    sensornames{sensorcount}=sensor{1};%#ok<AGROW> %eg 'bat'
    for irec=2:Ntot
        if ~strcmp(sensor{irec},sensor{irec-1})
            sensorcount=sensorcount+1;
            sensornames{sensorcount}=sensor{irec};%eg 'prs'
        end
        sensortag(irec)=sensorcount;
    end
    Nsens=sensorcount;
        
    %% Open a file for each sensor (create or append)
    
    for isens=1:Nsens
        
        writefilename=[datadir,sitecode,'_' sensornames{isens}  ...
            datestr(dateS,'_yyyymmdd')...
            datestr(dateE,'_yyyymmdd')...
            '.txt'];
        
        if idownloadchunk==0
            %First time around check if the file already exists and throw an error if it does!
            %Opt out available in case we're carefully adding to an old one
            %We can only get to this point if we've already forced it to download earlier on
            %so no need for auto option.
            if  ~isempty(dir(writefilename))
                disp(writefilename)
                reply = input('File already exists! Continue to append? Y/N [N]:','s');
                if ~strcmpi(reply,'Y')
                    error('Aborting')
                end
            end
        end        
        
        fid=fopen(writefilename,'a');
        for irec=1:Ntot
            if sensortag(irec)==isens
                fprintf(fid,'%f %s %s %s \n',C{1}(irec,:),C{2}(irec,:),C{3}(irec,:),C{4}{irec,:});
            end
        end
        fclose(fid);
    end
end

