function runQCfundemo

% RUNQCFUNDEMO Demonstrate use of QC function TIDEGAUGEQC
%
% Edit this function to select tide gauge and range of dates for
% demonstration. It will download data from the IOC site, attempt QC, and
% produce a number of figures, saved to the
% directory rootdatadir/sitecode/FIGS/
%
% For specific events, select an event for 'eventname'
% For a new event, edit the figures to include the new event
%
% RUNQCFUNDEMO calls TIDEGAUGEQC - more detailed help is available there,
% including a list of dependencies.
%
%
%   
% 
% =====Dependencies======:
% Matlab libraries, including statistics toolbox.
% PARSE_PV_PAIRS (from Matlab Central, John d'Errico)
% 
% Functions to read from the IOC website, and reload locally:
% readIOC.m	
% readIOCmeta.m
% loadlocalIOC.m
%
% Tide processing library 
% NOCtidefit_ctlwrapper.m 
% NOCtidepred_wrapper.m 
% NOCtidefit.m (from Simon Williams, NOC)
% NOCtidepred.m (from Simon Williams, NOC)
% phamp0fast.m (from Simon Williams, NOC)
% ufsetfast.m (from Simon Williams, NOC)
% vsetfast.m (from Simon Williams, NOC)
% longfindfast.m (from Simon Williams, NOC)	
% readctl.m (from Simon Williams, NOC)
% 
% Quality control and tide-related functions
% spike_detect_spline.m
% stuck_detect_test.m
% stuck_osc_detect_test.m
% getskewsurge.m
% channelsimilar.m
% channelbetter.m
%
% Averaging functions:
% daily2mon.m
% dailymeans.m
% hourly2daily.m
% hourlymeans.m	
% Nminmeans.m
% filltoregular.m
% filtto60min.m
%
% Plotting tools:
% patchbackgroundx.m
% savefigure.m	    
% plotwithgaps.m
%
% Other tools:
% modnear.m
% polyfitNoTests.m
% rounddatesec.m  
%
% Joanne Williams joll@noc.ac.uk March 2019


%% Define data directory, eg
rootdatadir='C:\users\fred\dataQC\data\';
%rootdatadir='/home/fred/dataQC/data/';

%% Select site and dates - uncomment or edit as required

%It is assumed that all dates are given in UTC. 
% Dates dowloaded are dateS <= date < dateE 
% Eg to download all data in 2013 
% dateS=datenum(2013,01,01);
% dateE=datenum(2014,01,01);
% (several of the following examples therefore miss out Dec 31st.)

%%Avonmouth Portbury %Avonmouth data deteriorates in 2018.
%sitecode='avon';
%sitename='Avonmouth';
%dateS=datenum(2012,01,01); %actually before first data on IOC site, which was 2012,09,04
%dateE=datenum(2019,01,24);

%%Lowestoft %Looks like it might be a good gauge for looking at skew surges
% sitecode='lowe';
% sitename='Lowestoft';
% dateS=datenum(2012,09,04); %First data
% dateE=datenum(2019,01,24);

%%Newlyn
% sitecode='newl2';
% sitename='Newlyn 2';
% dateS=datenum(2015,01,01);
% dateE=datenum(2019,01,24);

%%Calliaqua %Calliaqua has a big data gap, just picked up again. (But still not on IOC, Jan 2019?)
%  sitecode='calq';
%  sitename='Calliaqua';
% dateS=datenum(2013,12,12); %first record
% dateE=datenum(2014,12,13);
%%Caliaqua complete:
%   dateS=datenum(2013,12,12); %first record
%   dateE=datenum(2017,11,23); %data logger damage.
%%Caliaqua short test:
% dateS=datenum(2013,12,12); %first record
% dateE=datenum(2014,08,01); %data logger damage. Pressure channel suspect after datenum(2014,06,03)
%
%  dateS=datenum(2017,08,24); %very short test
%  dateE=datenum(2017,09,24);
%
% dateS=datenum(2017,01,01); %Just handling this year creates a problem as prs is drifting badly
% dateE=datenum(2017,12,31);


%%Prickly bay:
%  sitecode='pric';
%  sitename='Prickly Bay';
%  dateS=datenum(2017,01,01); % Saved in WEBSITE FOLDER %(THIS ONE IS SAVED IN WORKSHOP FOLDER)
%  dateE=datenum(2017,12,31);
%%Prickly bay test2: %includes some tricky problems with the pressure sensor.
%  dateS=datenum(2017,01,01);
%  dateE=datenum(2018,12,20);
%%Prickly bay complete:
% dateS=datenum(2011,05,14);
% dateE=datenum(2018,10,04);
%%Prickly bay test (particularly some tricky flags): Good for demo - see PricklyBayDemoPlots1
%dateS=datenum(2013,02,01);
%dateE=datenum(2013,07,01);


%%St Lucia test
 sitecode='stlu';
 sitename='Ganter''s Bay';
% dateS=datenum(2016,08,25);
 dateS=datenum(2018,01,01);
 dateE=datenum(2018,12,31);

%%St Martin (near Anguilla)
%  sitecode='stmt';
% sitename='St Martin';
%  dateS=datenum(2015,12,29);
%  dateE=datenum(2019,02,28);


%%Virgin Islands,  Charlotte-Amalie_VI
% sitecode='amal';
% sitename=' Charlotte-Amalie_VI';
%  dateS=datenum(2017,01,01);
%  dateE=datenum(2017,12,31);

%%La Desirade Island, Guadeloupe; desi;
% sitecode='desi';
% sitename='La Desirade Island, Guadeloupe';
%   dateS=datenum(2017,01,01);
%  dateE=datenum(2017,12,31);

%%St Louis Senegal
%This is a very good example of why you mustn't use too many constituents
%when fitting the tide.
% sitecode='stlo2';
% sitename='St Louis, Senegal';
% dateS=datenum(2016,02,10);
% dateE=datenum(2016,12,09);
%There is actually also some data in 2015, 2017 & March 2018 but it is very bad...
%it would be good to reject it automatically! Biggest problem is that the isolated scraps
%interfere with the tide.

%%Dakar (Senegal)
%Dakar is a long record and a good example of what happens when there's a
%datum shift. There are big shifts around 2011-12-10 and 2014-05-30.
%With default settings, shifts are not properly handled, so most of the data is
%wasted.
%The pressure sensor appears to have a consistent datum, but the radar
%sensors have big jumps.
%
%One approach to big jumps would be split the time series at an early
%(or iterative) stage so that they are handled separately.
%A quick fix is to set a datum correction to align a section of the data,
%but this requires manual intervention
%sitecode='dakar';
%sitename='Dakar';
%dateS=datenum(2007,10,27);
%dateE=datenum(2019,02,04);
%dateS=datenum(2007,10,27);
%dateE=datenum(2011,12,09);
% %dateS=datenum(2011,12,10);
% %dateE=datenum(2019,02,04);


%%Example of Cyclone Yemyin – hit Karachi (kara) on 25th / 26th of June 2007.
%%Surge should be about 1m.
% sitecode='kara';
% sitename='Karachi';
% % dateS=datenum(2007,01,01);
% % dateE=datenum(2008,01,01);
%  dateS=datenum(2007,05,09);
%  dateE=datenum(2008,05,09);


%%Examples from Chilean tsunami, 27th February 2010.
%
% sitecode='cald';
% sitename='Caldera';
%  dateS=datenum(2010,01,01);
%  dateE=datenum(2010,04,01);
%
% sitecode='corr';
% sitename='Corral';
%  dateS=datenum(2010,01,01);
%  dateE=datenum(2010,04,01);
%
% sitecode='coqu';
% sitename='Coquimbo';
%  dateS=datenum(2010,01,01);
%  dateE=datenum(2010,04,01);
%
% sitecode='valp';
% sitename='Valparaiso';
%  dateS=datenum(2010,01,01);
%  dateE=datenum(2010,04,01);


%%Indonesia
% sitecode='waik';
% sitename='Waikelo';
% dateS=datenum(2018,01,01); %NOT THE START!
% dateE=datenum(2019,02,13);
%
% sitecode='koli2';
% sitename='Kolinamil 2'; %Jakarta
% dateS=datenum(2018,01,01); %NOT THE START!
% dateE=datenum(2019,02,13);


%%Denmark
% sitecode='hirt';
% sitename='Hirtshals'; % Country 	Denmark
% % 2016-03-27T02:00:00 - Clocks change - record is in local winter/summertime!
% % 2016-10-30T02:00:00 - Clocks change - record is in local winter/summertime!
% %dateS=datenum(2016,01,01);
% %dateE=datenum(2016,04,01);
% dateS=datenum(2016,10,15);
% dateE=datenum(2017,01,01);

%Mtwara, Tanzania - Truncated sensor (half-tide) test
% the Pressure sensor doesn't record low water levels, but the radar sensor does.
% THIS IS NOT A ROBUST RESULT - for now reject half-tide sensors
% sitename='Mtwara'; % Tanzania
% sitecode='mtwa';
% dateS=datenum(2009,09,10);
% dateE=datenum(2019,03,13);


%%Liverpool (Gladstone Dock) - note that longer data will be available
% %elsewhere. Manual QC'd data is also available.
% sitecode='live';
% sitename='Liverpool';
% dateS=datenum(2012,09,04);
% dateE=datenum(2019,02,04);

%%Example of intermittently gappy data, with higher than 1 minute frequency
% sitename= 'Holyhead';
% sitecode='holy';
% dateS=datenum(2018,01,01);
% dateE=datenum(2019,03,13);


%% ===== PLOTS =========

%FLAGS SHOULD BE MADE CONSISTENT 1/0

%Plot lots of figures?
plotdemofigs=1;

%% ========= Select specific events to plot ============
% eventname=''; % Don't plot figures for any specific events
% eventname='Yemyin'; %Cyclone, Karachi 2017
eventname='IrmaMaria'; %Hurricanes, Caribbean
% eventname='Venezuela'; %Earthquake
% eventname='ChileTsunami'; %For the Chilean Earthquake, 2010
% eventname='Krakatau'; %For the Krakatau tsunami, 2018 %This doesn't show up well in the gauges on IOC
% eventname='DanishSummertime';
% eventname='DanishWintertime';
% eventname='Giovanna'; %Hurricane, Tanzania


%% === Certain sites have known problems with sensors - add that in here if required. ===
switch sitecode
    case {'calq'}
        if dateS>=datenum(2016,4,1)
            %excludesensors={'prs'};
            excludesensors={};
        else
            excludesensors={};
        end
        
    case 'valp'
        excludesensors={'rad'};
    case 'stlu'
        excludesensors={'pr2'};
    case 'waik'
        excludesensors={'enc'};
    otherwise
        excludesensors={};
end


%% Relative datums of the channels
% If specified, this is used instead of medians to compare channels for QC
% No absolute datums are specified.
% Channel numbers should match the sensornames list specified above.
% It is particularly important at Calliaqua, where one sensor has a severe
% drift compared to the others, so the median of the full series is not a
% useful measure.
% Set to empty if not known or required.

switch sitecode
    case 'calq'
        if dateS<datenum(2015,0,0)
            relativetochan1=[0,0,0]; %Calliaqua - all channels start off on the same datum
            %3 channels because it has to include the battery!
        else
            relativetochan1=[];
        end
   %case 'dakar'
   %    relativetochan1=[1.5230,6.5220,1.1210];  %Based on median of ~2018 (final year)
   %[1.6242]    [6.5641]    [1.1944] %Or based on "mostcommonmed"
        
    otherwise
        relativetochan1=[];
end


%% ================= END OF SETTINGS ================= %%


%% =======APPLY QC.===========
% Data will be written to rootdatadir/sitecode/
% and can then be retrieved to plot figures

tidegaugeQC(sitecode,dateS,dateE,...
    'sitename',sitename,...
    ...'usesensors',usesensors,...
    'excludesensors',excludesensors,...
    'relativetochan1',relativetochan1,...
    'rootdatadir',rootdatadir);

%% ===== Load data=========
datadir=[rootdatadir  sitecode '/'];

figdir=[datadir,'FIGS/'];
if ~exist(figdir,'dir')
    mkdir(figdir)
end

basefile=[sitecode, datestr(dateS,'_yyyymmdd'), datestr(dateE,'_yyyymmdd')];

load([datadir,basefile,'_sensornames.mat'],'sensornames','chanvec');%Something to say which sensors we have

%Note that the high-frequency heights/dates are not actually changed by the
%QC, there's just flagging.
load([datadir,basefile,'_reldatumestimate.mat'],'datumcor_afterqc');
load([datadir,basefile,'_highfreq.mat'],'datescell','predcell','heightscell',...
    'stuckflagscell','spikeflagscell','chanflagprefcell');
load([datadir,basefile,'_hourly.mat'],'dateshourlycell','predhourlycell','nontidalresidhourlycell','heightshourlycell');
load([datadir,basefile,'_slow_nodatum.mat'],'datesdaycell','heightsdaycell','datesmoncell','heightsmoncell');
load([datadir,basefile,'_skew_nodatum.mat'],'datesskewscell','maxTidecell','skewscell');

load([datadir,basefile,'_highfreq_stitched.mat'],'singlechanneldates','singlechannelpred','singlechannelheights','singlechannelntr','singlechannelorigin');
load([datadir,basefile,'_hourly_stitched.mat'],'singlechanneldateshourly','singlechannelpredhourly','singlechannelntrhourly','singlechannelheightshourly');
load([datadir,basefile,'_slow_nodatum_stitched.mat'],'singlechanneldatesday','singlechannelheightsday','singlechanneldatesmon','singlechannelheightsmon');
load([datadir,basefile,'_skew_nodatum_stitched.mat'],'singlechanneldatesskews','singlechannelmaxTide','singlechannelskews');

if ~isempty(eventname)
    ploteventfigs=1;
    
    switch eventname
        %To add an event, specify:
        %deventplotS, deventplotE : date limits for the plot
        %eventplottitle
        %deventS=[S], deventE=[E] : defines the start and end of an event
        %deventS=[S], deventE=[S] is OK for a short event (eg earthquake)
        %deventS=[S1,S2], deventE=[E1,E2] where there are 2 events (eg Irma & Maria)
        %eventlabel : indicated on plot.
        
        case 'Venezuela'
            %dates for Venezuela earthquake. Unfortunately *all* Caribbean
            %gauges examined have a data gap - presumably a comms problem.
            deventplotS=datenum(2018,08,20);
            deventplotE=datenum(2018,08,23);
            eventplottitle='Tsunami after August 2018 Earthquake?';
            deventS=[datenum(2018,08,21,21,0,0)];
            deventE=[datenum(2018,08,22,1,0,0)];
            eventlabel='Earthquake';
        case 'Honduras'
            %Honduras tsunami alert
            deventplotS=datenum(2018,01,10);
            deventplotE=datenum(2018,01,12);
            eventplottitle='Tsunami after Honduras 2018 Earthquake?';
            deventS=[]; deventE=[];
            eventlabel='Earthquake';
        case 'ChileTsunami'
            deventplotS=datenum(2010,02,26);
            deventplotE=datenum(2010,02,30);
            eventplottitle='Tsunami after Feb 2010 Earthquake';
            deventS=[datenum(2010,02,27,06,34,0)];
            deventE=[datenum(2010,02,27,06,34,0)];
            eventlabel='Earthquake';
        case 'Yemyin'
            %dates for Cyclone Yemyin
            deventplotS=datenum(2007,06,22);
            deventplotE=datenum(2007,06,28);
            eventplottitle='Cyclone Yemyin, 2007';
            deventS=[datenum(2007,06,25)];
            deventE=[datenum(2007,06,27)];
            eventlabel='Yemyin nearby';
        case 'Giovanna'
            %dates for Cyclone Giovanna
            deventplotS=datenum(2012,02,12);
            deventplotE=datenum(2012,02,16);
            eventplottitle='Cyclone Giovanna, 2012';
            deventS=[datenum(2012,02,13)];
            deventE=[datenum(2012,02,14)];
            eventlabel='Giovanna nearby';
        case 'IrmaMaria'
            %dates for Hurricane Irma & Maria
            deventplotS=datenum(2017,08,20);
            deventplotE=datenum(2017,09,20);
            eventplottitle='Hurricanes, 2017';
            deventS=[datenum(2017,09,04),datenum(2017,09,17)];
            deventE=[datenum(2017,09,07),datenum(2017,09,20)];
            eventlabel='Irma/Maria nearby';
        case 'Tomas'
            %dates for Hurricane Tomas
            deventplotS=datenum(2010,10,20);
            deventplotE=datenum(2010,11,10);
            eventplottitle='Hurricane Tomas, 2010';
            deventS=[datenum(2010,10,29)];
            deventE=[datenum(2010,10,31)];
            eventlabel='Tomas nearby';
        case 'Krakatau' %See http://www.gdacs.org/Public/download.aspx?type=DC&id=137
            %dates for Krakatau (including later eruptions)
            deventplotS=datenum(2018,12,20);
            deventplotE=datenum(2019,01,02);
            eventplottitle='Tsunami after Krakatau 2018?';
            eventlabel='';
            deventS=[]; deventE=[];
        case 'DanishSummertime'
            deventplotS=datenum(2016,03,26);
            deventplotE=datenum(2016,03,28);
            eventplottitle='Clocks in local time';
            deventS=[datenum(2016,03,27,02,0,0)];
            deventE=[datenum(2016,03,27,02,0,0)];
            eventlabel='Miss one hour';
        case 'DanishWintertime'
            deventplotS=datenum(2016,10,29);
            deventplotE=datenum(2016,10,31);
            eventplottitle='Clocks in local time';
            deventS=[datenum(2016,10,30,03,0,0)];
            deventE=[datenum(2016,10,30,03,0,0)];
            eventlabel='Repeat one hour';
    end
    
else
    ploteventfigs=0;
end


% Some figure colors:
colorlist        ={    [1,0,0],[0.5,0.8,0.2],    [0,0,1]};
darkcolorlist    ={0.5*[1,0,0],0.5*[0,1,0]  ,0.5*[0,0,1]};
palecolorlist    ={[1,0.5,0.5],[0.5,1,0.5]  ,[0.5,0.5,1]};
nearcolorlist    ={[1,0,1]    ,[1,1,0]      ,[0,1,1]    };
neardarkcolorlist={0.4*[1,0,1],0.4*[1,1,0]  ,0.4*[0,1,1]};

%% Plot raw data, indicating spikes & stuck data flags, and channels still suspect after ntr comparison

if plotdemofigs
    maxall=max(cell2mat(heightscell(:)));
    minall=min(cell2mat(heightscell(:)));
    
    figure
    for ichan=chanvec
        sensstr=sensornames{ichan}; %eg prs
        dates=datescell{ichan}(:);
        heights=heightscell{ichan};
        spikeflags=spikeflagscell{ichan};
        stuckflags=stuckflagscell{ichan};
        chanflagpref=chanflagprefcell{ichan};
        
        plot(dates,...
            heights,'color',colorlist{ichan},'displayname',['raw data ',sensstr])
        hold on
        plot(dates((spikeflags==4|stuckflags==4)&chanflagpref==1),...
            heights((spikeflags==4|stuckflags==4)&chanflagpref==1),'*','color',neardarkcolorlist{ichan},'displayname',['flags ',sensstr])
        hold on
        
        %Shade the suspect sections from badON to badOFF
        badON=dates(diff(chanflagpref)==2); %badON chanpref goes from 1 to 3
        badOFF=dates(diff(chanflagpref)==-2); %badON chanpref goes from 3 to 1
        ylim([minall,maxall]);
        patchbackgroundx(badON,badOFF,'facecolor',palecolorlist{ichan},'edgecolor','none','facealpha',0.5,'displayname',['suspect ' sensstr]);
        
        legend('off'),legend('show','location','best')
    end
    ylabel('metres')
    
    datetick('x',selectdateformat(gca),'keeplimits')
    title([sitename ', Flags and different channels'],'interpreter','none')
    legend('off'),legend('show','location','best')
    savefigure(gcf,[figdir,basefile,'_heights_raw'],[],'png300')
end


%% Plot total heights (Pref channels, unflagged data only, mark flags separately)

if plotdemofigs
    
    figure
    for ichan=chanvec
        sensstr=sensornames{ichan}; %eg prs
        dates=datescell{ichan};
        heights=heightscell{ichan};
        flags=(spikeflagscell{ichan}==4|stuckflagscell{ichan}==4);
        chanflagpref=chanflagprefcell{ichan};
        
        %Bad times in Channels flagged in chanflagprefcell are now set to heights= NaN
        heights(chanflagpref~=1)=nan;
        
        plotwithgaps(dates(~flags),heights(~flags),'color',colorlist{ichan},'displayname',['height ', sensstr]);
        hold on
        if sum(flags)>0
            plot(dates(flags),heights(flags),'*','color',neardarkcolorlist{ichan},'displayname',['flagged points'] );
        end
        
    end
    
    ylabel('metres')
    
 datetick('x',selectdateformat(gca),'keeplimits')
 title([sitename ', Auto Quality Controlled'],'interpreter','none')
    legend('off'),legend('show','location','best')
    savefigure(gcf,[figdir,basefile,'_heights_afterQC'],[],'png300')
    
end

%% Plot non-tidal residual (Pref channels, unflagged data only, mark flags separately)

if plotdemofigs
    
    
    figure
    for ichan=chanvec
        
        sensstr=sensornames{ichan}; %eg prs
        dates=datescell{ichan};
        heights=heightscell{ichan};
        pred=predcell{ichan};
        spikeflags=(spikeflagscell{ichan}==4);
        stuckflags=(stuckflagscell{ichan}==4);
        flags=spikeflags|stuckflags;
        chanflagpref=chanflagprefcell{ichan};
        
        %Bad times in Channels flagged in chanflagprefcell are now set to heights= NaN
        heights(chanflagpref~=1)=nan;
        
        plotwithgaps(dates(~flags),heights(~flags)-pred(~flags),'color',colorlist{ichan},'displayname',['non-tidal residual ', sensstr]);
        hold on
        if sum(spikeflags)>0
            plot(dates(spikeflags),heights(spikeflags)-pred(spikeflags),'*','color',neardarkcolorlist{ichan},'displayname',['flags'] );
        end
        %      if sum(stuckflags&idate&chanflagpref)>0
        %          plot(dates(stuckflags&idate),zeros(size(dates(stuckflags&idate))),'m.','displayname',['flags'] );
        %      end
        
    end
    
    ylabel('metres')
    
    
    ylabel('metres')
    
     datetick('x',selectdateformat(gca),'keeplimits')
    title([sitename ', Auto Quality Controlled'],'interpreter','none')
    legend('off'),legend('show','location','best')
    savefigure(gcf,[figdir,basefile,'_nontidalresidual_afterQC'],[],'png300')
    
end


%% Plot total heights for specific events

if ploteventfigs
    
    figure
    for ichan=chanvec
        sensstr=sensornames{ichan}; %eg prs
        dates=datescell{ichan};
        heights=heightscell{ichan};
        flags=(spikeflagscell{ichan}==4|stuckflagscell{ichan}==4);
        chanflagpref=chanflagprefcell{ichan};
        
        %Bad times in Channels flagged in chanflagprefcell are now set to heights= NaN
        heights(chanflagpref~=1)=nan;
        
        idate=dates>deventplotS&dates<deventplotE;
        
        
        plotwithgaps(dates(~flags&idate),heights(~flags&idate),'color',colorlist{ichan},'displayname',['height ', sensstr]);
        hold on
        if sum(flags&idate)>0
            plot(dates(flags&idate),heights(flags&idate),'g*','displayname',['flags'] );
        end
        
    end
    
    ylabel('metres')
    
    datetick('x',selectdateformat(gca),'keeplimits')
    patchbackgroundx(deventS,deventE,'displayname',eventlabel,...
        'facecolor',[0.9,0.2,0.5],'edgecolor',[0.9,0.2,0.5],...
        'facealpha',0.4,'edgealpha',0.4,'linewidth',1)
    
    title([sitename ', ' eventplottitle],'interpreter','none')
    legend('off'),legend('show','location','best')
    savefigure(gcf,[figdir,basefile,'_heights_' eventname],[],'png300')
    
    
end

%% Plot non-tidal residuals for specific events

if ploteventfigs
    
    figure
    for ichan=chanvec
        sensstr=sensornames{ichan}; %eg prs
        dates=datescell{ichan};
        heights=heightscell{ichan};
        pred=predcell{ichan};
        spikeflags=(spikeflagscell{ichan}==4);
        stuckflags=(stuckflagscell{ichan}==4);
        flags=spikeflags|stuckflags;
        chanflagpref=chanflagprefcell{ichan};
        
        %Bad times in Channels flagged in chanflagprefcell are now set to heights= NaN
        heights(chanflagpref~=1)=nan;
        
        
        idate=dates>deventplotS&dates<deventplotE;
        
        
        plotwithgaps(dates(~flags&idate),heights(~flags&idate)-pred(~flags&idate),'color',colorlist{ichan},'displayname',['non-tidal residual ', sensstr]);
        hold on
        if sum(spikeflags&idate)>0
            plot(dates(spikeflags&idate),heights(spikeflags&idate)-pred(spikeflags&idate),'g*','displayname',['flags'] );
        end
        %      if sum(stuckflags&idate&chanflagpref)>0
        %          plot(dates(stuckflags&idate),zeros(size(dates(stuckflags&idate))),'m.','displayname',['flags'] );
        %      end
        
    end
    
    ylabel('metres')
       datetick('x',selectdateformat(gca),'keeplimits')
   patchbackgroundx(deventS,deventE,'displayname',eventlabel,...
        'facecolor',[0.9,0.2,0.5],'edgecolor',[0.9,0.2,0.5],...
        'facealpha',0.4,'edgealpha',0.4,'linewidth',1)
            
    title([sitename ', ' eventplottitle],'interpreter','none')
    
    legend('off'),legend('show','location','best')
    savefigure(gcf,[figdir,basefile,'_ntr_' eventname],[],'png300')
    
    
end

%% Plot Hourly non-tidal residuals for specific events

if ploteventfigs
    
    
    figure
    for ichan=chanvec
        sensstr=sensornames{ichan}; %eg prs
        dateshourly=dateshourlycell{ichan}(:);
        nontidalresidhourly=nontidalresidhourlycell{ichan};
        
        idate=dateshourly>deventplotS&dateshourly<deventplotE;
        
        
        plotwithgaps(dateshourly(idate),nontidalresidhourly(idate),'color',colorlist{ichan},'displayname',['hourly non-tidal residual ', sensstr]);
        hold on
        
    end
    
    
    ylabel('metres')   
    datetick('x',selectdateformat(gca),'keeplimits')
    patchbackgroundx(deventS,deventE,'displayname',eventlabel,...
        'facecolor',[0.9,0.2,0.5],'edgecolor',[0.9,0.2,0.5],...
        'facealpha',0.4,'edgealpha',0.4,'linewidth',1)
    
    title([sitename ', ' eventplottitle],'interpreter','none')
    legend('off'),legend('show','location','best')
    savefigure(gcf,[figdir,basefile,'_hourly_' eventname],[],'png300')

end

%% Plot stitched Channel (note these will identical to the above if there is only one channel)
if plotdemofigs
    
    
    figure
    plot(singlechanneldateshourly,singlechannelpredhourly,'displayname',['predicted tide' ]);
    hold on
    plotwithgaps(singlechanneldates,singlechannelheights,'color','k','displayname',['heights , single channel' ]);
    
    ylabel('metres')
    
      datetick('x',selectdateformat(gca),'keeplimits')
    title([sitename ', Auto Quality Controlled, stitched single channel'],'interpreter','none')
    legend('off'),legend('show','location','best')
    savefigure(gcf,[figdir,basefile,'_heights_afterQC_stitched_tide'],[],'png300')
    
    
    
    
    figure
    plotwithgaps(singlechanneldates,singlechannelheights,'color','k','displayname',['heights , single channel' ]);
    hold on
    for ichan=chanvec
        sensstr=sensornames{ichan}; %eg prs
        plotwithgaps(singlechanneldates(singlechannelorigin==ichan),singlechannelheights(singlechannelorigin==ichan),...
            'color',colorlist{ichan},'displayname',['heights from '  sensstr]);
    end
    ylabel('metres')
    
      datetick('x',selectdateformat(gca),'keeplimits')
    title([sitename ', Auto Quality Controlled, stitched single channel'],'interpreter','none')
    legend('off'),legend('show','location','best')
    savefigure(gcf,[figdir,basefile,'_heights_afterQC_stitched'],[],'png300')
    
    
    
    figure
    plotwithgaps(singlechanneldates,singlechannelntr,...
        'color','k','displayname',['non-tidal residual , single channel' ]);
    hold on
    for ichan=chanvec
        sensstr=sensornames{ichan}; %eg prs
        plotwithgaps(singlechanneldates(singlechannelorigin==ichan),...
            singlechannelntr(singlechannelorigin==ichan),...
            'color',colorlist{ichan},'displayname',['non-tidal residual from ' sensstr ]);
    end
    ylabel('metres')
    
       datetick('x',selectdateformat(gca),'keeplimits')
    title([sitename ', Auto Quality Controlled, stitched single channel'],'interpreter','none')
    legend('off'),legend('show','location','best')
    savefigure(gcf,[figdir,basefile,'_nontidalresidual_afterQC_stitched'],[],'png300')
end


%% Plot Skew Surge


if plotdemofigs
    
    figure
    for ichan=chanvec
        sensstr=sensornames{ichan}; %eg prs
        datesskews=datesskewscell{ichan};
        skews=skewscell{ichan};
        
        plot(datesskews,skews,'x','color',colorlist{ichan}, 'displayname',['skewsurge ', sensstr] )
        hold on
        
    end
    
    legend('off'),legend('show','location','best')
      datetick('x',selectdateformat(gca),'keeplimits')
    ylabel('metres')
    title([sitename ', Skew Surges'],'interpreter','none')
    savefigure(gcf,[figdir,basefile,'_skewsurge'],[],'png300')
    
    
    figure
    
    plot(singlechanneldatesskews,singlechannelskews,'x','color','k', 'displayname',['skewsurge single channel'] )
    
    legend('off'),legend('show','location','best')
       datetick('x',selectdateformat(gca),'keeplimits')
    ylabel('metres')
    title([sitename ', Skew Surges, stitched single channel'],'interpreter','none')
    
    savefigure(gcf,[figdir,basefile,'_skewsurge_stitched'],[],'png300')
end

%% Plot Dailies and monthlies

if plotdemofigs
    
    figure
    for ichan=chanvec
        
        % Lazy reference to each channel
        sensstr=sensornames{ichan}; %eg prs
        datesday=datesdaycell{ichan};
        heightsday=heightsdaycell{ichan};
        datesmon=datesmoncell{ichan};
        heightsmon=heightsmoncell{ichan};
        
        plot(datesday,heightsday,'+','color',nearcolorlist{ichan},'markersize',10,'displayname',['daily Doodson ',sensstr])
        %    plot(datesday,heightsday,'+','markersize',10,'displayname',['daily usenanmean ',sensstr])
        
        hold on
        plot(datesmon,heightsmon,'p','color',colorlist{ichan},'markersize',10,'linewidth',3,'displayname',['monthly ',sensstr])
        
    end
    
    legend('off'),legend('show','location','best')
     datetick('x',selectdateformat(gca),'keeplimits')
    ylabel('metres')
    title([sitename ', Daily & Monthly means'],'interpreter','none')
    savefigure(gcf,[figdir,basefile,'_dailymeans_originaldatum'],[],'png300')
    
    
    % Plot on (approximately) the same datum for each channel
    
    figure
    for ichan=chanvec
        
        % Lazy reference to each channel
        sensstr=sensornames{ichan}; %eg prs
        datesday=datesdaycell{ichan};
        heightsday=heightsdaycell{ichan};
        datesmon=datesmoncell{ichan};
        heightsmon=heightsmoncell{ichan};
        
        plot(datesday,heightsday-datumcor_afterqc{ichan},'+','color',nearcolorlist{ichan},'markersize',10,'displayname',['daily Doodson ',sensstr])
        %    plot(datesday,heightsday,'+','markersize',10,'displayname',['daily usenanmean ',sensstr])
        
        hold on
        plot(datesmon,heightsmon-datumcor_afterqc{ichan},'p','color',colorlist{ichan},'markersize',10,'linewidth',3,'displayname',['monthly ',sensstr])
        
    end
    
    legend('off'),legend('show','location','best')
      datetick('x',selectdateformat(gca),'keeplimits')
    ylabel('metres')
    title([sitename ', Daily & Monthly means, datum adjusted'],'interpreter','none')
    savefigure(gcf,[figdir,basefile,'_dailymeans'],[],'png300')
    
    % Plot single channel:
    
    figure
    plot(singlechanneldatesday,singlechannelheightsday,'+','color',nearcolorlist{1},'markersize',10,'displayname',['daily Doodson single channel'])
    %    plot(datesday,heightsday,'+','markersize',10,'displayname',['daily usenanmean ',sensstr])
    hold on
    plot(singlechanneldatesmon,singlechannelheightsmon,'p','color',colorlist{1},'markersize',10,'linewidth',3,'displayname',['monthly  single channel'])
    
    legend('off'),legend('show','location','best')
      datetick('x',selectdateformat(gca),'keeplimits')
    ylabel('metres')
    title([sitename ', Daily & Monthly means'],'interpreter','none')
    savefigure(gcf,[figdir,basefile,'_dailymeans_originaldatum_stitched'],[],'png300')
    
    
    
end




