% SPIKE_DETECT_SPLINE
%
% Spike detection function
%
% Usage: flags=spike_detect_spline(heights,dates,splineparam,verbosity)
%
% This version is a function, accepts data and parameters for spline
%
% splineparam.deviationmethod : 'rmse', {'median'} : compare using rmse or median absolute deviation
% splineparam.timedelta:{1} sample interval in minutes
% splineparam.Nwin :    {60} Number of data of the window to fit the spline
% splineparam.nsigma:   {4} Number of sigmas to identify an event (only used for rmse method)
% splineparam.nMAD:     {6} Number of scaled median absolute deviations to identify an event (only used for median method)
% splineparam.minspike: {0} Minimum deviation to identify an event (only used for median method)
% splineparam.ndegree:  {2} Degree of the spline
%
% One problem that remains is that if there are a number of outliers close
% together, they will likely affect the spline. A quadratic fitted using
% least squares (as in polyfit) is not robust against an outlier.
% A robust method for fitting the quadratic would be better, but it is very
% likely to severely impact performance.
%
% Instead, it is recommended to run the spike detection (at least) twice,
% removing the spikes after each pass.
%
% option 'verbose' results in spike figures being plotted though not saved.
%
% Flags returned are 1 for GOOD data, 4 for BAD data (consistent with EuroGOOS)
% 
% ---------------
% Code history: 
% Based on python code from Begona Perez Gomez, Created on Oct 2017 %  author: EPPE\bpg
% 
% Converted to Matlab June 2018 and optimised by Joanne Williams  
% 
% Edited 9 Jan 2019 (Joanne Williams): change test from >= to >. Otherwise constant
% heights (or where heights are exactly equal to a spline) will also be
% reported as a SPIKE.
% If the instrument is stuck it should return a STUCK flag instead.
%
% Edited 1st Feb 2019 (Joanne Williams): allow option for median absolute deviation instead
% of RMSE difference. This is a more robust measure of the deviation.
% Also the point being tested is excluded from the spline fit.
% --------------------
%
% This function is required for TIDEGAUGEQC.
% 
% Joanne Williams, Feb 2019, joll@noc.ac.uk


function flags=spike_detect_spline(heights,dates,splineparam,verbosity)

if nargin<2
    error('function requires heights and dates')
end

%   Parameters to be fixed outside (properties file)
if nargin<3||isempty(splineparam)
    splineparam=[];
end

if isfield(splineparam,'deviationmethod')
    deviationmethod=splineparam.deviationmethod;
else
    %deviationmethod='rmse'; %original method
    deviationmethod='median'; %Now defaults to median
end

if isfield(splineparam,'timedelta') %Unused.
    timedelta=splineparam.timedelta;
else
    timedelta=1; %in minutes
end

if isfield(splineparam,'Nwin')
    Nwin=splineparam.Nwin; %this would be better as a time interval?
else
    Nwin=60;      %  Number of data of the window to fit the spline
end

if isfield(splineparam,'nsigma')
    nsigma=splineparam.nsigma;
else
    nsigma = 4;    %  Number of sigmas to identify an event (only used for rmse method)
end

if isfield(splineparam,'nMAD')
    nMAD=splineparam.nMAD;
else
    nMAD = 6;    %  Number of scaled median absolute deviations to identify an event (only used for median method)
end

if isfield(splineparam,'minspike')
    minspike=splineparam.minspike;
else
    minspike = 0;    %  Minimum deviation to identify an event (only used for median method)
end


if isfield(splineparam,'ndegree')
    ndegree=splineparam.ndegree; %Degree of the spline
else
    ndegree=2;         %  Degree of the spline
end


if nargin==4&&strcmp(verbosity,'verbose')
    verbose=1;
    figure
    plot(dates,heights)
    hold on
else
    verbose=0;
end

if Nwin==60&&(dates(2)-dates(1))>=2/(24*60)
    warning('Not 1 minute data - is the spline parameter ok?')
end

switch deviationmethod
    case 'median'
        medianscaling=-1/(sqrt(2)*erfcinv(3/2));
end


flags=nan(size(dates));
%Although the following changes the size of heights and dates, it shouldn't
%matter as they aren't returned, and flags is filled carefully.
inan=isnan(heights)|isnan(dates);
heights(inan)=[];
dates(inan)=[];

ndatmax = length(dates);							   %  Number of data
ifl = zeros(size(dates));

%% Loop through data to fit spline
%definition of a window (j1 to j2)for each individual i value 

ws = warning('off','all'); %Temporarily silence warnings.
for i=1:ndatmax %1:ndatmax
    
    if (i-1 < Nwin/2) %first section
        j1=1;
        j2=Nwin;
        %ifit = i; % ifit was the position of the fitted point relative to the window but it is now explicitly excluded from the window
    elseif (i-1 >= ndatmax - Nwin/2)  %last section
        j1=ndatmax-Nwin;
        j2=ndatmax;
        %ifit = (Nwin)-(j2-i)+1;
    else  %middle
        j1 = (i - Nwin/2);
        j2 = (i + Nwin/2);
        %ifit =(Nwin/2+1) ;
        
        
        
    end
    
    %datewin = dates([j1:j2]);
    %Datawin = heights([j1:j2]);              %  Data window for i data value
    
    %Explicitly exclude the value itself from the fit
    %CHECK - does this risk throwing out the genuine maximum?
    
    datewin = dates([j1:i-1,i+1:j2]);
    Datawin = heights([j1:i-1,i+1:j2]);              %  Data window for i data value
    
    
    
    datemu1=(dates(j2)+dates(j1))/2;% dates(i);
    datemu2=(dates(j2)-dates(j1))/3;
    %Speed up from polyfit by cutting out condition number tests and
    %manually centering
    %     [z,~,mu] = polyfitNoTests(datewin, Datawin, ndegree) ;  %  Spline fit of degree "ndegree": array with polynomial coefficients
    %     yfit     = polyval(z,datewin,[],mu); %mu provides centering and scaling
    
    %centering: use estimate datemu1 datemu2 instead of mean, std; Only time needs centering.
    
    %Note that the polynomial fit is not robust to outliers - though the
    %risk should be reduced by excluding the point being tested.
    %A robust fit could be used instead but this would be substantially slower
    
    [z] = polyfitNoTests((datewin-datemu1)/datemu2, Datawin, ndegree) ;  %  Spline fit of degree "ndegree": array with polynomial coefficients
    
    yfit     = polyval(z,(datewin-datemu1)/datemu2); %mu provides centering and scaling
    yifit=polyval(z,(dates(i)-datemu1)/datemu2); %mu provides centering and scaling
    
    
    switch deviationmethod
        case 'rmse'
            rmse_val = sqrt(mean((yfit-Datawin).^2));     %  Computes rmse fit/real data in the window
            
            if (abs(yifit-heights(i)) > nsigma*rmse_val)       %  Difference i-value/yfit(i), flagged if larger than nsigma*rmse
                iflag=4;
                ifl(i)=iflag;
            else
                iflag=1;
                ifl(i)=iflag;
            end
        case 'median' %Median absolute deviation
            %Definition of scaled median absolute deviation is taken from
            %Matlab help file for ISOUTLIER (R2017a)
            A=(yfit-Datawin);
            scaledMAD=medianscaling*median(abs(A-median(A)));
            if abs(yifit-heights(i))>nMAD*scaledMAD... %default nMAD=6
                    &&abs(yifit-heights(i))>minspike %allows minimum based on typical data spike
                iflag=4;
                ifl(i)=iflag;
            else
                iflag=1;
                ifl(i)=iflag;
            end
        otherwise
            error('splineparam.deviationmethod must be ''rmse'' or ''median'' ')
    end
    
    if verbose
        if iflag==4
            
            plot(datewin,Datawin)
            hold on
            plot(datewin,yfit)
            plot(dates(i),heights(i),'*')
            
        end

    %    DemoPlotsPric1.m
    end
    
    
end

%Restore warnings
warning(ws);

if verbose
    figure
    plot(dates,heights,'.-','displayname','recorded heights')
    hold on
    plot(dates(ifl==4),heights(ifl==4),'*','displayname','spikes found')
end

%Now populate flags correctly, avoiding nan spaces
flags(~inan)=ifl;


